﻿using System;

namespace Seg7ClockControler
{
	/* コマンド定義
	 * 下位4bit：命令の長さ(byte)
	 * 8bit全体として：命令の種類
	 *（例）0x17 … 命令番号=17、命令の長さ=7byte
	 */
	struct CMD
	{
		public const int
			SET_DATE		= 0x17,		//日時を設定する
			SET_HOURMODE	= 0x21,		//時間制を設定する
			SET_CON_EFF		= 0x32,		//コントラスト、更新エフェクトを設定する
			GET_VALUE		= 0x40,		//現在の設定値を取得する
			BUF_CLEAR		= 0x00;		//受信バッファをクリアする
	};
	
	/*
	 * 7Seg Clock
	 */
	class Seg7Clock
	{
		private CdcIO cdcio;
		public int dispMode, contrast, effect;
		
		//コンストラクタ
		public Seg7Clock()
		{
			cdcio = null;
			dispMode = contrast = effect = 0;
		}
		
		//CdcIOオブジェクトをセットする
		public void setCdcIO(CdcIO cdcio)
		{
			this.cdcio = cdcio;
		}

		//日時を設定する
		public bool setDate(DateTime dt)
		{
			bool ret = cdcio.sendByte(CMD.SET_DATE)
					&& cdcio.sendByte(dt.Year % 100)	//年（西暦の下2桁）
					&& cdcio.sendByte(dt.Month)		//月
					&& cdcio.sendByte(dt.Day)		//日
					&& cdcio.sendByte((int)dt.DayOfWeek)	//曜日（0:日,1:月～6:土）
					&& cdcio.sendByte(dt.Hour)		//時(24時間制)
					&& cdcio.sendByte(dt.Minute)	//分
					&& cdcio.sendByte(dt.Second);	//秒
			return ret;
		}

		//表示モード（時間制）を設定する
		public bool setDisplayMode(int mode)
		{
			//0:12時間制, 1:24時間制, 2:30時間制
			if (mode < 0 || 2 < mode) return false;

			bool ret = cdcio.sendByte(CMD.SET_HOURMODE)
					&& cdcio.sendByte(mode);
			
			if (ret) dispMode = mode;
			return ret;
		}

		//コントラスト、更新エフェクトを設定する
		public bool setConEff(int con, int eff)
		{
			bool ret = cdcio.sendByte(CMD.SET_CON_EFF)
					&& cdcio.sendByte(con)		//コントラスト		0 - 1
					&& cdcio.sendByte(eff);		//更新エフェクト	0(off) / 1(on)
			
			if (ret)
			{
				contrast = con;
				effect = eff;
			}
			return ret;
		}

		//現在の設定値を取得する
		public bool getValue()
		{
			if (!cdcio.sendByte(CMD.GET_VALUE)) return false;
			
			int value = cdcio.receiveByte();
			if (cdcio.cmdGetError) return false;
			dispMode = value;

			value = cdcio.receiveByte();
			if (cdcio.cmdGetError) return false;
			contrast = value;

			value = cdcio.receiveByte();
			if (cdcio.cmdGetError) return false;
			effect = value;
			
			return true;
		}
		
	}
	
}
